@@ -114,7 +114,7 @@ static Translator* NewTranslator(void)
 {//===================================
 	Translator *tr;
 	int ix;
-	static const unsigned char stress_amps2[] = {17,17, 20,20, 20,22, 22,20 };
+	static const unsigned char stress_amps2[] = {18,18, 20,20, 20,22, 22,20 };
 	static const short stress_lengths2[8] = {182,140, 220,220, 220,240, 260,280};
 	static const wchar_t empty_wstring[1] = {0};
 	static const wchar_t punct_in_word[2] = {'\'', 0};  // allow hyphen within words
@@ -308,14 +308,17 @@ void SetIndicLetters(Translator *tr)
 	memset(tr->letter_bits,0,sizeof(tr->letter_bits));
 	SetLetterBitsRange(tr,LETTERGP_A,0x04,0x14);   // vowel letters
 	SetLetterBitsRange(tr,LETTERGP_A,0x3e,0x4d);   // + vowel signs, and virama
+	SetLetterBitsRange(tr,LETTERGP_A,0x55,0x57);   // + vowel signs
 
 	SetLetterBitsRange(tr,LETTERGP_B,0x3e,0x4d);   // vowel signs, and virama
+	SetLetterBitsRange(tr,LETTERGP_B,0x55,0x57);   // + vowel signs
 
 	SetLetterBitsRange(tr,LETTERGP_C,0x15,0x39);   // the main consonant range
 	SetLetterBits(tr,LETTERGP_C,dev_consonants2);  // + additional consonants
 
 	SetLetterBitsRange(tr,LETTERGP_Y,0x04,0x14);   // vowel letters
 	SetLetterBitsRange(tr,LETTERGP_Y,0x3e,0x4c);   // + vowel signs
+	SetLetterBitsRange(tr,LETTERGP_Y,0x55,0x57);   // + vowel signs
 
 	tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
 	tr->langopts.suffix_add_e = tr->letter_bits_offset + 0x4d;   //virama
@@ -343,7 +346,7 @@ Translator *SelectTranslator(const char *name)
 	static const short stress_lengths_sk[8] = {190,190, 210,210, 0,0, 210,210};
 
 	static const short stress_lengths_ta[8] = {200, 200,  210, 210,  0, 0,  230, 230};
-	static const unsigned char stress_amps_ta[8] = {18,18, 18,18, 20,20, 22,22 };
+	static const unsigned char stress_amps_ta[8] = {18,18, 18,18, 20,20, 21,21 };
 
 	// convert name string into a word of up to 4 characters, for the switch()
 	while(*name != 0)
@@ -458,13 +461,14 @@ Translator *SelectTranslator(const char *name)
 	case L('d','e'):
 		{
 			static const short stress_lengths_de[8] = {150,130, 200,200,  0, 0, 270,270};
+			static const unsigned char stress_amps_de[] = {20,20, 20,20, 20,22, 22,20 };
+			SetupTranslator(tr, stress_lengths_de, stress_amps_de);
 			tr->langopts.stress_rule = STRESSPOSN_1L;
 			tr->langopts.word_gap = 0x8;   // don't use linking phonemes
 			tr->langopts.vowel_pause = 0x30;
 			tr->langopts.param[LOPT_PREFIXES] = 1;
 			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x10;  // devoice at end of word
 			tr->langopts.param[LOPT_LONG_VOWEL_THRESHOLD] = 175/2;
-			memcpy(tr->stress_lengths,stress_lengths_de,sizeof(tr->stress_lengths));
 		
 			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_SWAP_TENS | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_ALLOW_SPACE | NUM_ORDINAL_DOT | NUM_ROMAN;
 			SetLetterVowel(tr,'y');
@@ -1089,6 +1093,30 @@ SetLengthMods(tr,3);  // all equal
 		}
 		break;
 
+	case L('s','i'):  // Sinhala
+		{
+			SetupTranslator(tr,stress_lengths_ta,stress_amps_ta);
+			tr->langopts.length_mods0 = tr->langopts.length_mods;  // don't lengthen vowels in the last syllable
+
+			tr->langopts.stress_rule = STRESSPOSN_1L;
+			tr->langopts.stress_flags =  S_FINAL_DIM | S_NO_AUTO_DIM | S_FINAL_NO_2;
+			tr->langopts.spelling_stress = 1;
+
+			tr->letter_bits_offset = OFFSET_SINHALA;
+			memset(tr->letter_bits,0,sizeof(tr->letter_bits));
+			SetLetterBitsRange(tr,LETTERGP_A,0x05,0x16);   // vowel letters
+			SetLetterBitsRange(tr,LETTERGP_A,0x4a,0x73);   // + vowel signs, and virama
+		
+			SetLetterBitsRange(tr,LETTERGP_B,0x4a,0x73);   // vowel signs, and virama
+		
+			SetLetterBitsRange(tr,LETTERGP_C,0x1a,0x46);   // the main consonant range
+		
+			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
+			tr->langopts.suffix_add_e = tr->letter_bits_offset + 0x4a;   //virama
+			tr->langopts.numbers =  NUM_OMIT_1_THOUSAND ;
+		}
+		break;
+
 	case L('s','l'):  // Slovenian
 			tr->charset_a0 = charsets[2];   // ISO-8859-2
 			tr->langopts.stress_rule = STRESSPOSN_2R;   // Temporary
@@ -1222,7 +1250,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = 7;   // stress on the last syllable, before any explicitly unstressed syllable
 			tr->langopts.stress_flags = 0x20;  //no automatic secondary stress
 
-			tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA | NUM_ALLOW_SPACE | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_2;
+			tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_2;
 			tr->langopts.max_initial_consonants = 2;
 		}
 		break;
